﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using DG.Tweening;
using System.Linq;

public class AudioMenago : MonoBehaviour {
	static public AudioMenago instance = null;

	public AudioClip deathScreen;
	public AudioClip explosion;
	public AudioClip microwaveClose;
	public AudioClip microwaveExplosion;
	public AudioClip microwaveLoop;
	public AudioClip microwaveOpen;
	public AudioClip microwaveOpenClose;
	public AudioClip overcookingLoop;
	public AudioClip perfect;
	public AudioClip startGame;
	public AudioClip timeLow;
	public AudioClip waveMatch;

	public enum NegativeAudio
	{
		None = -1,
		JaZaToNieZaplaceA = 0,
		JaZaToNieZaplaceB,
		CoZaGownoA,
		CoZaGownoB,
		JaNieBedeTegoJadA,
		JaNieBedeTegoJadB,
		MAX
	};

	public enum PositiveAudio
	{
		None = -1,
		WunderbarA = 0,
		PalceLizacA,
		NieboWGebieA,
		PrawdziwyRarytasA,
		NazarlemSieJakSwiniaA,
		NiciZMojejDietyA,
		MAX
	};

	PositiveAudio lastRolledPositive = PositiveAudio.None;
	NegativeAudio lastRolledNegative = NegativeAudio.None;

	public List<AudioClip> positive = new List<AudioClip> ();
	public List<AudioClip> negative = new List<AudioClip> ();

//	List<AudioSource> audioSourcesMusic = new List<AudioSource>();
	float volumeMultiplier = 1.0f;
	float volumeVoiceMultiplier = 1.0f;
	List<AudioSource> audioSourcesSfx = new List<AudioSource>();

	Dictionary<string, AudioSource> tagsAudioSources = new Dictionary<string, AudioSource>();

	// Use this for initialization
	void Start () {
		instance = this;
	}
	
	// Update is called once per frame
	void Update () {
		foreach (AudioSource audio in audioSourcesSfx) {
			Debug.Log ("isPlaying " + audio.isPlaying);
		}
	}

	AudioSource getAudioSourceForSfx()
	{
		// get audiosoure
		AudioSource retVal = null;
		foreach (AudioSource audio in audioSourcesSfx) {
			if (audio.isPlaying == false) {
				retVal = audio;
				break;
			}
		}
		if (retVal == null) {
			retVal = gameObject.AddComponent<AudioSource> ();
			audioSourcesSfx.Add (retVal);
		}

		// standard setup
		retVal.loop = false;
		retVal.volume = 1.0f * volumeMultiplier;

		return retVal;
	}

	static public AudioSource GetSourceForTag(string tag)
	{
		AudioSource retVal = null;
		instance.tagsAudioSources.TryGetValue (tag, out retVal);
		return retVal;
	}

	static public AudioSource PlaySfx(AudioClip clip, float volume = 1.0f, float fadeInDuration = -1.0f)
	{
		AudioSource audio = instance.getAudioSourceForSfx ();
		audio.clip = clip;
		audio.Play ();

		if (fadeInDuration == -1.0f) {
			audio.volume = volume * instance.volumeMultiplier;
		} else {
			audio.volume = 0.0f;
			audio.DOFade (volume * instance.volumeMultiplier, fadeInDuration);
		}

		return audio;
	}

	static public PositiveAudio RollNewPositiveAudio()
	{
		List<int> ids = Enumerable.Range (0, (int)PositiveAudio.MAX).ToList ();
		if (instance.lastRolledPositive != PositiveAudio.None) {
			ids.RemoveAt ((int)instance.lastRolledPositive);
		}
		instance.lastRolledPositive = (PositiveAudio)ids [Random.Range (0, ids.Count)];
		return instance.lastRolledPositive;
	}

	static public NegativeAudio RollNewNegativeAudio()
	{
		List<int> ids = Enumerable.Range (0, (int)NegativeAudio.MAX).ToList ();
		if (instance.lastRolledNegative != NegativeAudio.None) {
			ids.RemoveAt ((int)instance.lastRolledNegative);
		}
		instance.lastRolledNegative = (NegativeAudio)ids [Random.Range (0, ids.Count)];
		return instance.lastRolledNegative;
	}

	static public AudioSource PlayPositiveVoice(PositiveAudio index, float volume = 1.0f, float delay = 0.0f)
	{
		AudioSource audio = instance.getAudioSourceForSfx ();
		audio.clip = instance.positive [(int)index];
		if (delay > 0.0f) {
			audio.PlayScheduled (AudioSettings.dspTime + delay);
		} else {
			audio.Play ();
		}
		audio.volume = volume * instance.volumeVoiceMultiplier;

		Sequence seq = DOTween.Sequence ();
		seq.AppendInterval (delay + 0.5f);
		seq.AppendCallback (() => {
			voiceoversubtitles.Instance.SpawnNewText ((int)index, true);
		});

		return audio;
	}

	static public AudioSource PlayNegativeVoice(NegativeAudio index, float volume = 1.0f, float delay = 0.0f)
	{
		AudioSource audio = instance.getAudioSourceForSfx ();
		audio.clip = instance.negative [(int)index];
		if (delay > 0.0f) {
			audio.PlayScheduled (AudioSettings.dspTime + delay);
		} else {
			audio.Play ();
		}
		audio.volume = volume * instance.volumeVoiceMultiplier;

		Sequence seq = DOTween.Sequence ();
		seq.AppendInterval (delay + 0.5f);
		seq.AppendCallback (() => {
			voiceoversubtitles.Instance.SpawnNewText ((int)index, false);
		});

		return audio;
	}

	static public AudioSource PlaySfxLoop(AudioClip clip, string tag, float volume = 1.0f, float fadeInDuration = -1.0f)
	{
		AudioSource audio = PlaySfx (clip, volume, fadeInDuration);
		audio.loop = true;
		instance.tagsAudioSources.Add (tag, audio);

		return audio;
	}

	static public void StopSfxLoop(string tag)
	{
		AudioSource audio = GetSourceForTag (tag);
		if (audio) {
			audio.Stop ();
			instance.tagsAudioSources.Remove (tag);
		}
	}

	static public void SetTagVolume(string tag, float volume, float fadeDuration = -1.0f)
	{
		AudioSource audio = GetSourceForTag (tag);
		if (audio) {
			audio.DOComplete ();
			if (fadeDuration == -1.0f) {
				audio.volume = volume * instance.volumeMultiplier;
			} else {
				audio.DOFade (volume * instance.volumeMultiplier, fadeDuration);
			}
		}
	}

	static public void FadeOutAndStopAllAudio(float duration)
	{
		instance.tagsAudioSources.Clear ();
		foreach (AudioSource audio in instance.audioSourcesSfx) {
			audio.DOComplete (true);
			Sequence seq = DOTween.Sequence ();
			seq.Append(audio.DOFade (0.0f, duration));
			seq.AppendCallback (() => {
				audio.Stop();
			});
		}
	}

	static public bool IsVolumeFading(string tag)
	{
		AudioSource audio = GetSourceForTag (tag);
		if (audio) {
			return DOTween.IsTweening (audio);
		}
		return false;
	}
}
