﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class Food : MonoBehaviour {

	public List<Texture> foods 			= new List<Texture> ();
	public List<Texture> foodsFrozen 	= new List<Texture> ();
	public List<Texture> foodsExploded 	= new List<Texture> ();

	public Renderer quadRenderer;
	public Renderer quadRenderer2;

	public int requiredPitch = 0;
	float mass = 1;
	float explodeProgress = 0.8f;

	float maxForce = 0.1f;
	float slowdownForce = -0.2f;

	bool isCooking = true;
	float progress = 0.0f;
	float acceleration = 0.0f;
	float speed = 0.0f;

	int foodID = 0;

	TextMesh textTM;

	// Use this for initialization
	void Start () {
		textTM = transform.FindChild ("text").GetComponent<TextMesh> ();
		
	}

	public void SetParameters(int foodID, int pitch, float mass, float price)
	{
		this.foodID = foodID;
		this.requiredPitch = pitch;
		this.mass = mass;
		this.explodeProgress = Random.Range (0.85f, 0.95f);

		TargetController.SetTargetFrequency ((Waveplay.GetPitchRanges (requiredPitch) [0] + Waveplay.GetPitchRanges (requiredPitch) [1]) / 2.0f);
		AudioMenago.SetTagVolume ("waveMatch", 0.0f, 0.3f);
		LightController.SetActive (false);
		AudioMenago.SetTagVolume ("overcookingLoop", 0.0f, 0.3f);

		quadRenderer.material.SetTexture ("_MainTex", foodsFrozen [foodID]);
		quadRenderer2.material.SetTexture ("_MainTex", foods [foodID]);
		quadRenderer2.material.SetColor("_Color", new Color (1.0f, 1.0f, 1.0f, 0.0f));
		//float newWidth = Mathf.Lerp (0f, 18.0f, (float)foodsFrozen [foodID].height / 830.0f);
		//quadRenderer.transform.localScale = new Vector3 (newWidth * foodsFrozen [foodID].width/foodsFrozen [foodID].height, newWidth, 1.0f);
	}

	// Update is called once per frame
	void Update () {
		AudioMenago.SetTagVolume ("waveMatch", 0.0f);
		if (isCooking) {
			// speed based on input
			if (Waveplay.GetCurrentPitch () == requiredPitch && Waveplay.GetCurrentLoudness() > 0.08f) {
				if(Waveplay.GetCurrentCenterness() > 0.5f)
				{
					AudioMenago.SetTagVolume ("waveMatch", (Waveplay.GetCurrentCenterness()-0.5f) * 2.0f);
				}
				if (Waveplay.GetCurrentCenterness () > 0.8f) {
					LightController.SetActive (true);
				} else {
					LightController.SetActive (false);
				}
				acceleration = (maxForce / mass) * Waveplay.GetCurrentCenterness() * Waveplay.GetCurrentLoudness();
			} else {
				AudioMenago.SetTagVolume ("waveMatch", 0.0f);
				LightController.SetActive (false);
				acceleration = (slowdownForce / mass);
			}
			speed += acceleration * Time.deltaTime;
			speed = Mathf.Max (-0.02f, speed);
			// update progress
			progress += speed * Time.deltaTime;
			progress = Mathf.Max (0.0f, progress);

			Termometer.SetLocked (speed > 0.0f);
			Termometer.SetProgress (progress);

			quadRenderer.material.SetColor("_Color", new Color (1.0f, 1.0f, 1.0f, 1.0f));
			if (progress > explodeProgress) {
				quadRenderer.material.SetColor("_Color", new Color (1.0f, 1.0f, 1.0f, 0.0f));
				if (exploded == false)
					transform.position = Vector3.zero;
					Explode ();
				}
			else if (progress > 0.8f) {
				quadRenderer.material.SetColor("_Color", new Color (1.0f, 1.0f, 1.0f, 0.0f));
				quadRenderer2.material.SetColor ("_Color", Color.Lerp (Color.white, new Color (1.0f, 0.5f, 0.0f), (progress-0.8f)/(explodeProgress-0.8f)));	
				AudioMenago.SetTagVolume ("overcookingLoop", (progress - 0.8f) / (explodeProgress - 0.8f));

				Camera.main.transform.position = new Vector3 (0, 0, -10) + 0.5f*(Random.onUnitSphere * ((progress - 0.8f) / (explodeProgress - 0.8f)));
				transform.position = Vector3.zero + (Random.onUnitSphere * ((progress - 0.8f) / (explodeProgress - 0.8f)));
				transform.position = new Vector3 (transform.position.x, transform.position.y, 0.0f);
				overlaycontroller.Instance.rend.color = Color.Lerp (new Color(1,0,0,0), new Color (1, 0,0,0.15f), (progress - 0.8f) / (explodeProgress - 0.8f));
			} else if (progress >= 0.5f) {
				Camera.main.transform.position = new Vector3 (0, 0, -10);
				quadRenderer2.material.SetTexture ("_MainTex", foods [foodID]);
				quadRenderer2.material.SetColor ("_Color", Color.Lerp (new Color (1.0f, 1.0f, 1.0f, 0.0f), Color.white, (progress-0.5f)/0.2f));	
			}
		}

	}

	public int getFoodID()
	{
		return foodID;
	}

	public float getProgress()
	{
		return progress;
	}

	public bool didExplode()
	{
		return exploded;
	}

	public bool IsCooling()
	{
		return speed <= 0.0f;
	}

	public void FinishCooking()
	{
		Debug.Log ("finished cooking");
		isCooking = false;
		speed = 0.0f;
		Termometer.SetLocked (false);
	}

	bool exploded = false;
	public void Explode ()
	{
		if (exploded) {
			return;
		}
		overlaycontroller.Instance.rend.color = new Color (1, 0, 0, 0);

		exploded = true;
		FinishCooking ();
		quadRenderer2.material.SetTexture ("_MainTex", foodsExploded [foodID]);
		MicrowaveBGController.IncreaseDirtness ();
		ExplodeAnimController.Instance.PlayAnimation ();
		TargetController.SetTargetFrequency (0);

		LightController.SetActive (false);
		AudioMenago.SetTagVolume ("waveMatch", 0.0f, 0.3f);
		AudioMenago.SetTagVolume ("overcookingLoop", 0.0f, 0.3f);
		AudioMenago.PlaySfx (AudioMenago.instance.explosion);

		Waveplay.OnExplosion ();
	}

}
