﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using DG.Tweening;
using System.Linq;
using UnityEngine.SceneManagement;

public class Waveplay : MonoBehaviour {

	static public Waveplay instance = null;

	public SpriteRenderer splash;

	List<KeyValuePair<float, float>> foodsParams = new List<KeyValuePair<float, float>>()
	{
		new KeyValuePair<float, float>(5.0f, 0.5f), // frytki
		new KeyValuePair<float, float>(8.0f, 0.8f), // hotdog
		new KeyValuePair<float, float>(10.0f, 1.0f), // hamburger
		new KeyValuePair<float, float>(12.0f, 1.2f), // pizza
		new KeyValuePair<float, float>(15.0f, 2.0f)  // superburger
	};

	int lastRolledFoodID = -1;
	int lastRolledPitch = -1;

	enum State {
		Initial,
		Starting,
		CookingFood,
		RemovingFood,
		InsertingFood,
		Finished,
		FinishedWaitingForRestart,
		Restarting
	};

	State state = State.Initial;

	int currentPitch = -1;
	float currentPitchCenterness = 0.0f;
	float currentLoudness = 0.0f;

	int explosionsCounter = 0;

	float timeCurrent = 0.0f;
	float timeTotal = 60.0f;

	public GameObject foodPrefab;
	public GameObject currentFoodGO = null;

	float highScore = 142.0f;

	float[][] ranges = new float[5][]
	{
		new float[] { 50, 150 },
		new float[] { 150, 250 },
		new float[] { 250, 350 },
		new float[] { 350, 450 },
		new float[] { 450, 550 },
	};

	// Use this for initialization
	void Start () {
		instance = this;
		TargetController.SetTargetFrequency (0);
	}
	
	// Update is called once per frame
	void Update ()
	{

		if (state == State.RemovingFood || state == State.InsertingFood || state == State.CookingFood) {
			timeCurrent += Time.deltaTime;
			AudioMenago.SetTagVolume ("microwaveLoop", currentLoudness);
			FuckingLight.SetProgress (currentLoudness);
		}

		if (timeCurrent > timeTotal - 10.0f)
		{
			if (!AudioMenago.IsVolumeFading ("timeLow")) {
				Debug.Log ("timer");
				AudioMenago.SetTagVolume ("timeLow", 1.0f, 0.3f);
			}
		}
		
		Timer.SetProgress (Mathf.Clamp (timeCurrent / timeTotal, 0.0f, 1.0f));

		if (state != State.Finished
		    && state != State.FinishedWaitingForRestart
		    && state != State.Restarting
		    && timeCurrent >= timeTotal) {
			EndGameplay ();
		}

		if (state != State.Finished) {
			switch (state) {
			case State.Initial:
				{
					if (Input.GetKeyDown (KeyCode.Space) || Input.GetMouseButtonDown(0)) {
						StartGameplay ();
					}
					break;
				}
			case State.Starting:
				{
					// nop
					break;
				}
			case State.RemovingFood:
				{
					if (Input.GetKeyUp (KeyCode.Space) || Input.GetMouseButtonDown(0)) {
						StartInsertingFood ();
					}
					break;
				}
			case State.InsertingFood:
				{
					break;
				}
			case State.CookingFood:
				{
					if ((Input.GetKeyDown (KeyCode.Space) || Input.GetMouseButtonUp(0)) && currentFoodGO.GetComponent<Food> ().IsCooling ()) {
						StartRemovingFood ();
					}
					break;
				}
			case State.FinishedWaitingForRestart:
				{
					if (Input.GetKeyDown (KeyCode.Space) || Input.GetMouseButtonUp(0)) {
						state = State.Restarting;
						Sequence seq = DOTween.Sequence ();
						splash.transform.position = new Vector3 (splash.transform.position.x, 20, splash.transform.position.z);
						seq.Append (splash.transform.DOMoveY (0, 0.3f).SetEase (Ease.OutCubic));
						seq.AppendCallback (() => {
							SceneManager.LoadScene ("vapor");
						});
					}
					break;
				}
			case State.Restarting:
				{
					// nop
					break;
				}
			}

			RefreshCurrentPitch ();
		}
	}


	void StartGameplay()
	{
		if (state == State.Starting) {
			return;
		}
		state = State.Starting;

		AudioMenago.PlaySfx (AudioMenago.instance.startGame);
		AudioMenago.PlaySfxLoop (AudioMenago.instance.microwaveLoop, "microwaveLoop", 0.0f);
		AudioMenago.PlaySfxLoop (AudioMenago.instance.overcookingLoop, "overcookingLoop", 0.0f);
		AudioMenago.PlaySfxLoop (AudioMenago.instance.waveMatch, "waveMatch", 0.0f);
		AudioMenago.PlaySfxLoop (AudioMenago.instance.timeLow, "timeLow", 0.0f);

		LightController.SetActive (false);

		explosionsCounter = 0;
		timeCurrent = 0.0f;
		StartInsertingFood();

		splash.transform.DOMoveY (-20, 0.3f).SetEase (Ease.InCubic);
	}

	void EndGameplay()
	{
		if (state == State.Finished) {
			return;
		}
		state = State.Finished;

		LightController.SetActive (false);
		FuckingLight.SetProgress (0.0f);

		AudioMenago.SetTagVolume ("timeLow", 0.0f, 0.3f);
		AudioMenago.PlaySfx (AudioMenago.instance.deathScreen);

		Sequence seq1 = DOTween.Sequence ();
		seq1.AppendInterval (1.0f);
		seq1.AppendCallback (() => {
			highScore = Mathf.Max(highScore, MoneyText.GetMoney ());
			EndScreen.SetScores (MoneyText.GetMoney (), highScore);

			AudioMenago.FadeOutAndStopAllAudio (0.5f);
			AudioMenago.PlaySfx (AudioMenago.instance.startGame);

			Sequence seq2 = DOTween.Sequence ();
			seq2.AppendInterval (0.5f);
			seq2.AppendCallback (() => {
				state = State.FinishedWaitingForRestart;
				if(currentFoodGO) {
					Destroy(currentFoodGO);
				}
			});
		});

	}

	void StartRemovingFood()
	{
		if(state == State.RemovingFood) {
			return;
		}
		state = State.RemovingFood;

		if (currentFoodGO) {
			// reward
			RewardFood ();

			// animate out
			GameObject foodGoToDestroy = currentFoodGO;
			currentFoodGO = null;
			Sequence seq = DOTween.Sequence ();
			seq.Append (foodGoToDestroy.transform.DOMove (foodGoToDestroy.transform.localPosition + new Vector3 (30.0f, 0.0f, 0.0f), 0.3f, true).SetEase (Ease.OutBack));
			seq.AppendCallback (() => {
				Destroy (foodGoToDestroy);
			});

			AudioMenago.PlaySfx (AudioMenago.instance.microwaveOpen);
		}
	}

	void StartInsertingFood()
	{
		if(state == State.InsertingFood) {
			return;
		}
		state = State.InsertingFood;

		currentFoodGO = (GameObject)Instantiate (foodPrefab, new Vector3 (-30.0f, 0.0f, 0.0f), Quaternion.identity);

		// set parameter
		Food foodBeh = currentFoodGO.GetComponent<Food> ();
		List<int> foodsIds = Enumerable.Range (0, 5).ToList ();
		if (lastRolledFoodID != -1) {
			foodsIds.RemoveAt (lastRolledFoodID);
		}
		lastRolledFoodID = foodsIds [Random.Range (0, foodsIds.Count)];

		List<int> pitches = Enumerable.Range (0, 5).ToList ();
		if (lastRolledPitch != -1) {
			pitches.RemoveAt (lastRolledPitch);
		}
		lastRolledPitch = pitches [Random.Range (0, pitches.Count)];
		foodBeh.SetParameters (lastRolledFoodID, lastRolledPitch, foodsParams [lastRolledFoodID].Value, foodsParams [lastRolledFoodID].Key);

		MicControl.counter = -66;

		// animate in
		Sequence seq = DOTween.Sequence ();
		seq.Append (currentFoodGO.transform.DOMove (Vector3.zero, 0.3f, true).SetEase (Ease.OutBack, 3.0f));
		seq.AppendCallback (() => {
			StartCooking ();
		});

		AudioMenago.PlaySfx (AudioMenago.instance.microwaveClose);
	}

	void RewardFood()
	{
		Food foodBeh = currentFoodGO.GetComponent<Food> ();
		float money = 0.0f;
		if (!foodBeh.didExplode ()) {
			if (foodBeh.getProgress () >= 0.65f) {
				money += foodsParams [foodBeh.getFoodID ()].Key;
				if (foodBeh.getProgress () >= 0.7 && foodBeh.getProgress () < 0.75f) {
					// prefect meal bonus
					money *= 2.0f;

					Sequence delayed = DOTween.Sequence ();
					delayed.AppendInterval (0.2f).AppendCallback (() => {
						OverlayTextManager.Instance.Spawn5Sekund ();
					});

					AudioMenago.PositiveAudio aud = AudioMenago.RollNewPositiveAudio ();
					AudioMenago.PlayPositiveVoice (aud);

					timeCurrent = Mathf.Max (0.0f, timeCurrent - 5.0f);

					if (timeCurrent < timeTotal - 10.0f) {
						AudioMenago.SetTagVolume ("timeLow", 0.0f, 0.3f);
					}

					AudioMenago.PlaySfx (AudioMenago.instance.perfect);
				} else {
					AudioMenago.PlaySfx (AudioMenago.instance.startGame);
				}
			}
		}
		MoneyText.ChangeMoneycount ((int)money);

		if (money == 0)
		{
			AudioMenago.NegativeAudio aud = AudioMenago.RollNewNegativeAudio ();
			AudioMenago.PlayNegativeVoice (aud);
		}
	}

	void StartCooking()
	{
		if (state == State.CookingFood) {
			return;
		}
		state = State.CookingFood;
	}

	void RefreshCurrentPitch()
	{
		float freq = MicControl.GetSmoothedFrequency ();
		float volume = MicControl.GetSmoothedVolume ();

		currentPitch = -1;
		currentLoudness = Mathf.Clamp(volume * 4.0f, 0.0f, 1.0f);
		currentPitchCenterness = 0.0f;

		for (int i = 0; i < ranges.Length; ++i) {
			if (freq >= ranges [i] [0] && freq < ranges [i] [1]) {
				currentPitch = i;
				float rangeHalfWidth = (ranges [i] [1] - ranges [i] [0]) / 2.0f;
				float rangeCenter = ranges [i][0] + rangeHalfWidth;
				float distanceFromCenter = Mathf.Abs(rangeCenter - freq);
				currentPitchCenterness = 1.0f - distanceFromCenter / rangeHalfWidth;
			}
		}
	}

	public static void OnExplosion()
	{
		++instance.explosionsCounter;
		if (instance.explosionsCounter >= 3) {
			instance.EndGameplay ();
			AudioMenago.PlaySfx (AudioMenago.instance.microwaveExplosion, 1.0f, 0.0f);
		}
	}

	public static float[] GetPitchRanges(int pitch)
	{
		return instance.ranges [pitch];
	}

	public static int GetCurrentPitch()
	{
		return instance.currentPitch;
	}

	public static float GetCurrentLoudness()
	{
		return instance.currentLoudness;
	}

	public static float GetCurrentCenterness()
	{
		return instance.currentPitchCenterness;
	}
}
